import ReactDOM from 'react-dom/client'
import { getOrientation, isOs } from "../utils"
import React, { useRef, useState } from 'react';
import { TransitionGroup, CSSTransition } from "react-transition-group";
import '../App.less'
import { MessageInstance } from 'antd/es/message/interface';
import { CheckCircleFilled, CloseCircleFilled } from '@ant-design/icons';

let os = isOs()

// 创建容器
const el = document.createElement("div");
document.body.append(el);
el.className = 'my-message'

const root = ReactDOM.createRoot(el);

// 用于暴露 Notifications 里面的方法
const notificationRef: any = React.createRef();

const NotificationItem: React.FC<{ content: string, onRemove: any, type: 'SUCCESS' | 'ERROR' }> = ({ content, onRemove, type }) => {
    // 如果鼠标在上面就不要关闭通知
    const [isHover, setIsHover] = useState(false);

    React.useEffect(() => {
        let timeout: any;
        if (!isHover) {
            timeout = setTimeout(() => {
                // 通知父组件移除自身
                onRemove();
            }, 3000);
        }

        return () => {
            clearTimeout(timeout);
        };
    }, [isHover, onRemove]);

    return (
        <div
            onMouseEnter={() => setIsHover(true)}
            onMouseLeave={() => setIsHover(false)}
            className='message-content'
            style={{
                fontFamily: "-apple-system,BlinkMacSystemFont,'Segoe UI',Roboto,'Helvetica Neue',Arial,'Noto Sans',sans-serif,'Apple Color Emoji','Segoe UI Emoji','Segoe UI Symbol','Noto Color Emoji"
            }}
        >
            <div>{type === 'SUCCESS' ? <CheckCircleFilled style={{ color: '#52c41a' }} /> : type === 'ERROR' ? <CloseCircleFilled style={{ color: '#ff4d4f' }} /> : null}</div>{content?.split('').map((item, index) => <span key={index}>{item}</span>)}
        </div>
    );
};

const Notifications = React.forwardRef((_props, ref) => {
    // 通知列表
    const [list, setList] = useState<any[]>([]);

    // 通知自增 key
    const incrementKeyRef = useRef(0);

    // 这个 Hook 可以设置 Ref 的值，参考：https://reactjs.org/docs/hooks-reference.html#useimperativehandle
    React.useImperativeHandle(ref, () => ({
        notify(content: string, type: 'SUCCESS' | 'ERROR') {
            // 自增 key
            const key = incrementKeyRef.current++;
            setList((list) => {
                // 动画使用的是 React Transition Group
                const noti = (
                    <CSSTransition
                        key={key}
                        timeout={300}
                        classNames="message"
                        className="message"
                    >
                        <NotificationItem
                            onRemove={() => { setList((list) => { return list.filter((item: any) => item.key !== key.toString()) }) }}
                            content={content}
                            type={type}
                        />
                    </CSSTransition>
                );
                const newList = [...list, noti];
                return newList;
            });
        }
    }), []);

    return <TransitionGroup>{list}</TransitionGroup>;
});

// 渲染容器
root.render(<Notifications ref={notificationRef} />);

const showMessage = {
    success: (tips: string, message: MessageInstance) => {
        if (['ios', 'android'].includes(os) && getOrientation() === 'VERTICAL') {
            notificationRef.current.notify(tips, 'SUCCESS');
        } else {
            message.success(tips)
        }
    },
    error: (tips: string, message: MessageInstance) => {
        if (['ios', 'android'].includes(os) && getOrientation() === 'VERTICAL') {
            notificationRef.current.notify(tips, 'ERROR');
        } else {
            message.error(tips)
        }
    }
}

export default showMessage